from django.contrib import admin

from django_rtk_password import models


class PasswordResetTokenAdmin(admin.ModelAdmin):
    readonly_fields = ["token"]


admin.site.register(models.PasswordResetToken, PasswordResetTokenAdmin)



def request_password_reset_mutation(email, output_values):
    return """mutation {{
        requestPasswordReset(
            email: "{email}"
        ) {{
            {output_values}
        }}
    }}""".format(
        email=email,
        output_values=", ".join(output_values),
    )


def reset_password_mutation(password_reset_token, password, output_values):
    return """mutation {{
        resetPassword(
            passwordResetToken: "{password_reset_token}"
            password: "{password}"
        ) {{
            {output_values}
        }}
    }}""".format(
        password_reset_token=password_reset_token,
        password=password,
        output_values=", ".join(output_values),
    )


def change_password_mutation(email, password, new_password, output_values):
    return """mutation {{
        changePassword(
            email: "{email}"
            password: "{password}"
            newPassword: "{new_password}"
        ) {{
            {output_values}
        }}
    }}""".format(
        email=email,
        password=password,
        new_password=new_password,
        output_values=", ".join(output_values),
    )



import pytest
from django.contrib.auth import get_user_model
from django.test import Client
from django_rtk_password.models import PasswordResetToken
from django_rtk_password.tests.mutations import (
    change_password_mutation,
    request_password_reset_mutation,
    reset_password_mutation,
)

email = "tester@test.com"
password = "test1234"


class TestSchema:
    @pytest.fixture()
    def client(self):
        return Client()

    @pytest.fixture()
    def user_account(self):
        user = get_user_model().objects.create_user(
            email=email, password=password, accepts_terms=True
        )
        user.is_active = True
        user.save()
        return user

    @pytest.mark.django_db()
    def test_reset_password(self, client: Client, user_account):
        query = request_password_reset_mutation(
            email=email, output_values=["success", "errors"]
        )
        response = client.post("/graphql/", dict(query=query))

        # check that the response is as expected
        response_data = response.json()["data"]["requestPasswordReset"]
        assert response_data == {"success": True, "errors": {}}

        password_reset_token = PasswordResetToken.objects.get(email=email)

        query = reset_password_mutation(
            password_reset_token=password_reset_token.token,
            password="foobarbaz456",
            output_values=["success", "errors"],
        )
        response = client.post("/graphql/", dict(query=query))

        # check that the response is as expected
        assert response.json() == {
            "data": {"resetPassword": {"success": True, "errors": {}}}
        }

        query = change_password_mutation(
            email=email,
            password="foobarbaz456",
            new_password="foobarbaz789",
            output_values=["success", "errors"],
        )
        response = client.post("/graphql/", dict(query=query))

        # check that the response is as expected
        assert response.json() == {
            "data": {"changePassword": {"success": True, "errors": {}}}
        }

    @pytest.mark.django_db()
    def test_change_password_to_bad_password(self, client: Client, user_account):
        query = change_password_mutation(
            email="tester@test.com",
            password="foobarbaz123",
            new_password="foo",
            output_values=["success", "errors"],
        )
        response = client.post("/graphql/", dict(query=query))

        # check that the response is as expected
        assert response.json() == {
            "data": {
                "changePassword": {
                    "success": False,
                    "errors": {"password": ["TOO_SHORT"]},
                }
            }
        }

    @pytest.mark.django_db()
    def test_dangerously_expose_password_reset_token(
        self, client: Client, settings, user_account
    ):
        query = request_password_reset_mutation(
            email="tester@test.com",
            output_values=["passwordResetToken"],
        )

        response = client.post("/graphql/", dict(query=query))
        assert not response.json()["data"]["requestPasswordReset"]["passwordResetToken"]

        settings.DJANGO_RTK["DANGEROUSLY_EXPOSE_TOKENS"] = True
        response = client.post("/graphql/", dict(query=query))
        assert response.json()["data"]["requestPasswordReset"]["passwordResetToken"]
        settings.DJANGO_RTK["DANGEROUSLY_EXPOSE_TOKENS"] = False

    @pytest.mark.django_db()
    def test_hide_account_existence(self, client: Client, settings, user_account):
        query = request_password_reset_mutation(
            email="idontexist@test.com",
            output_values=["errors"],
        )

        response = client.post("/graphql/", dict(query=query))
        assert response.json() == {"data": {"requestPasswordReset": {"errors": {}}}}

        settings.DJANGO_RTK["HIDE_ACCOUNT_EXISTENCE"] = False
        response = client.post("/graphql/", dict(query=query))
        assert response.json() == {
            "data": {"requestPasswordReset": {"errors": {"email": ["ACCOUNT_UNKNOWN"]}}}
        }
        settings.DJANGO_RTK["HIDE_ACCOUNT_EXISTENCE"] = True



import graphene

from .changepassword import ChangePassword
from .requestpasswordreset import RequestPasswordReset
from .resetpassword import ResetPassword


class Mutation(graphene.ObjectType):
    request_password_reset = RequestPasswordReset.Field()
    reset_password = ResetPassword.Field()
    change_password = ChangePassword.Field()



import django_rtk.mutations as mutations
import graphene
from django_rtk.utils import (
    extract_token,
    get_backend,
    get_errors,
    get_setting_or,
    get_validator,
)


class RequestPasswordReset(mutations.RequestPasswordReset):
    class Arguments:
        email = graphene.String()

    password_reset_token = graphene.String()

    @classmethod
    def validate_args(cls, errors, email, **kwargs):
        get_validator().validate_email(errors, email)

    @classmethod
    def run(cls, errors, **kwargs):
        result = get_backend().request_password_reset(errors, **kwargs)

        hide_account_existence = get_setting_or(True, "HIDE_ACCOUNT_EXISTENCE")
        if hide_account_existence and "ACCOUNT_UNKNOWN" in get_errors(errors, "email"):
            get_errors(errors, "email").remove("ACCOUNT_UNKNOWN")

        return result

    @classmethod
    def get_output_values(cls, result):
        return {"password_reset_token": extract_token(result, "password_reset_token")}

    @classmethod
    def send_email(cls, result, email, **kwargs):
        if result.get("password_reset_token"):
            mutations.send_password_reset_email(result, email, **kwargs)



import uuid

from django.contrib.auth import authenticate, get_user_model
from django_rtk.utils import count_errors, get_errors

from django_rtk_password.models import PasswordResetToken


class Backend:
    def request_password_reset(self, errors, email, **kwargs):
        result = dict(password_reset_token="")
        if count_errors(errors):
            return result

        if not get_user_model().objects.filter(email=email).exists():
            get_errors(errors, "email").append("ACCOUNT_UNKNOWN")
            return result

        password_reset_token, _ = PasswordResetToken.objects.get_or_create(email=email)
        result["password_reset_token"] = password_reset_token.token.hex

        return result

    def reset_password(self, errors, password_reset_token, password, **kwargs):
        result = dict(user=None)
        if count_errors(errors):
            return result

        password_reset_token = PasswordResetToken.objects.filter(
            token=uuid.UUID(password_reset_token)
        ).first()

        if not password_reset_token:
            get_errors(errors, "password_reset_token").append("NOT_FOUND")
            return result

        user = result["user"] = (
            get_user_model().objects.filter(email=password_reset_token.email).first()
        )

        if user:
            user.set_password(password)
            user.save()
            password_reset_token.delete()
        else:
            get_errors(errors, "password_reset_token").append("ACCOUNT_UNKNOWN")

        return result

    def change_password(self, errors, email, password, new_password, **kwargs):
        result = dict(user=None)
        if count_errors(errors):
            return result

        user = get_user_model().objects.filter(email=email).first()
        if not user:
            get_errors(errors, "email").append("ACCOUNT_UNKNOWN")
            return result

        username = getattr(user, get_user_model().USERNAME_FIELD)
        is_authenticated = authenticate(username=username, password=password)
        if not is_authenticated:
            get_errors(errors, "password").append("INVALID_CREDENTIALS")
            return result

        user.set_password(new_password)
        user.save()
        return result



import django_rtk.mutations as mutations
import graphene
from django_rtk.utils import get_validator


class ResetPassword(mutations.ResetPassword):
    class Arguments:
        password_reset_token = graphene.String()
        password = graphene.String()

    @classmethod
    def validate_args(cls, errors, password, **kwargs):
        get_validator().validate_password(errors, password)
